<?php
namespace App\Factory;

use App\Exception;

class IntegrationFactory {
    private $connection;

    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function disableIntegration() {
        $sql = "
            UPDATE INTEGRACAO
               SET IDHABINTE    = 'N'
             WHERE IDINTEGRACAO = 'IFO'
        ";
        
        $this->connection->executeQuery($sql);
    }

    public function getConfiguration() {
        $sql = "
            SELECT IDUFTIMEZONE, IDTOKENINTGERAL, IDSENHATOKENGERAL, IDHABINTE, IDRASTREIAENTREGA, IDUTLVNDPDRPALC, AUTHTOKEN, AUTHTKNDTCRIAC, CDINTEGRACAO
              FROM INTEGRACAO
             WHERE IDINTEGRACAO = 'IFO'
        ";
        
        return $this->connection->fetchAssoc($sql);
    }

    public function getMerchantConfiguration($cdFilial) {
        $sql = "
            SELECT CDFILIAL, CDLOJA, CDCAIXA
              FROM INTEGRACAOFILI 
             WHERE CDINTEGRACAO = (SELECT CDINTEGRACAO FROM INTEGRACAO WHERE IDINTEGRACAO = :IDINTEGRACAO)
               AND CDFILIAL     = :CDFILIAL
        ";
        
        $params = [
            ":IDINTEGRACAO" => "SHU",
            ":CDFILIAL"     => $cdFilial
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function updateAccessToken($token, $time) {
        $sql = "
            UPDATE INTEGRACAO
               SET AUTHTOKEN      = :AUTHTOKEN,
                   AUTHTKNDTCRIAC = :AUTHTKNDTCRIAC
             WHERE IDINTEGRACAO   = :IDINTEGRACAO
        ";
    	
    	$params = [
    	    ":AUTHTOKEN"      => $token,
            ":AUTHTKNDTCRIAC" => $time,
            ":IDINTEGRACAO"   => 'IFO'
    	];
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function getOrderForCancel($orderReference) {
        $sql = "
        	SELECT C.CDFILIAL BRANCHID, C.NRCOMANDA ORDERNUMBER, C.NRVENDAREST SELLNUMBER
        	
        	  FROM COMANDAVEN C 

              JOIN VENDAREST VR
                ON VR.NRVENDAREST = C.NRVENDAREST
               AND VR.CDFILIAL    = C.CDFILIAL

         LEFT JOIN VENDA V
                ON C.CDFILIAL    = V.CDFILIAL
               AND C.NRVENDAREST = V.NRVENDAREST
               AND V.NRSEQVENDA IS NULL

            WHERE C.NRPEDMOBILE = :NRPEDMOBILE
              AND C.IDSTCOMANDA NOT IN ('4', 'X')
              AND C.IDORGCMDVENDA = 'DLV_IFO'
              AND VR.DTHRABERMESA >= SYSDATE - 1
         ORDER BY VR.DTHRABERMESA DESC
	    ";

        $params = [
            ':NRPEDMOBILE' => "IFO".$orderReference    
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function cancelOrder($cancelOrder) {
        $sql = "
            UPDATE COMANDAVEN
		       
		       SET IDSTCOMANDA = '4'
		     
		     WHERE CDFILIAL     = :CDFILIAL
	           AND NRCOMANDA    = :NRCOMANDA
	           AND NRVENDAREST  = :NRVENDAREST
	    ";
        
		$params = [
			':CDFILIAL'	   => $cancelOrder["BRANCHID"], 
			':NRCOMANDA'   => $cancelOrder["ORDERNUMBER"],
			':NRVENDAREST' => $cancelOrder["SELLNUMBER"]
		];
		
		$this->connection->executeQuery($sql, $params);
	}
	
	public function syncOrder($orderDetails) {
        $sql = "
	        UPDATE VENDAREST
		       
		       SET IDSINCMSDE   = 'N'
	         WHERE CDFILIAL     = :CDFILIAL
	           AND NRVENDAREST  = :NRVENDAREST
	    ";
        
        $params = [
            ':CDFILIAL'    => $orderDetails["BRANCHID"],
            ':NRVENDAREST' => $orderDetails["SELLNUMBER"]
        ];
       
        $this->connection->executeQuery($sql, $params);
	}
	
}
