<?php
namespace App\Factory;

use App\Exception;

class OrderFactory {

    private $connection;

    public function __construct (\Doctrine\DBAL\Connection $connection)
    {
        $this->connection = $connection;
    }
    
    public function insertSale($sale, $generalParams, $order, $consumer)
    {
        $sql = "
            INSERT INTO VENDAREST
    			(CDFILIAL, NRVENDAREST, CDLOJA, NRMESA, CDVENDEDOR,
    			DTHRABERMESA, DTHRFECHMESA, CDOPERADOR, NRPESMESAVEN,
    			CDCLIENTE, CDCONSUMIDOR, IDPEDIDOPAGO)
    		VALUES
    			(:CDFILIAL, :NRVENDAREST, :CDLOJA, '',
    			:CDVENDEDOR, TO_DATE(:CURRDATE, 'dd/mm/yyyy hh24:mi:ss'), null, :CDOPERADOR,
    			'1', :CDCLIENTE, :CDCONSUMIDOR, 'S')";
    			
    	date_default_timezone_set($this->getCurrentTimezone($generalParams["SGESTADO"]));
    	
    	$params = [
    	    ":CDFILIAL"     => $generalParams["CDFILIAL"],
            ":NRVENDAREST"  => $sale,
            ":CDLOJA"       => $generalParams["CDLOJA"],
            ":CDVENDEDOR"   => $generalParams["CDVENDEDOR"],
            ":CURRDATE"     => date("d/m/Y H:i:s"),
            ":CDOPERADOR"   => $generalParams["CDOPERADOR"],
            ":CDCLIENTE"    => $consumer["CDCLIENTE"],
            ":CDCONSUMIDOR" => $consumer["CDCONSUMIDOR"]    
    	];
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function insertSaleCommand($generalParams, $order, $consumer, $removeDeliveryFee)
    {
        $sql = "
            INSERT INTO COMANDAVEN 
                (CDFILIAL, NRVENDAREST, NRCOMANDA, NRCOMANDAEXT, 
    	        CDLOJA, DSCOMANDA, IDSTCOMANDA, SGSEXOCON, 
    	        TXMOTIVCANCE, VRACRCOMANDA, CDPAIS, SGESTADO, 
    	        CDMUNICIPIO, NRCEPCONSCOMAND, CDBAIRRO, DSBAIRRO, NRENDECONSCOMAN,
    	        DSENDECONSCOMAN, DSCOMPLENDCOCOM, DSREFENDCONSCOM, DSOBSCOMANDA,
    	        IDORGCMDVENDA, IDRETBALLOJA, NRPEDMOBILE, NRLATPEDIDO, NRLONGPEDIDO, 
    	        DSAREAATEND, IDSINCAGENDA, DTHRAGENDADA, IDCOMANDAEXT, IDENTRPLAT,
    	        NRCONTATOCON, DSCONSUMIDOR, IDIMPCUPEXP)
    	    VALUES 
    	        (:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRCOMANDAEXT, 
    	        :CDLOJA, :DSCOMANDA, :IDSTCOMANDA, :SGSEXOCON, 
    	        :TXMOTIVCANCE, :VRACRCOMANDA, :CDPAIS, :SGESTADO, 
    	        :CDMUNICIPIO, :NRCEPCONSCOMAND, :CDBAIRRO, :DSBAIRRO, :NRENDECONSCOMAN,
    	        :DSENDECONSCOMAN, :DSCOMPLENDCOCOM, :DSREFENDCONSCOM, :DSOBSCOMANDA,
    	        :IDORGCMDVENDA, :IDRETBALLOJA, :NRPEDMOBILE, :NRLATPEDIDO,
    	        :NRLONGPEDIDO, :DSAREAATEND, :IDSINCAGENDA, TO_DATE(:DTHRAGENDADA, 'dd/mm/yyyy hh24:mi:ss'),
    	        :IDCOMANDAEXT, :IDENTRPLAT, :NRCONTATOCON, :DSCONSUMIDOR, :IDIMPCUPEXP)";
    	
    	if($order["orderType"] === "DELIVERY" || $order["orderType"] === "DEFAULT") {
    	    $IDRETBALLOJA = "N";
    	    
    	    if(strtoupper($order["delivery"]["deliveryAddress"]["state"]) === "XX"){
    	        $order["delivery"]["deliveryAddress"]["state"] = 'AC';
    	        $order["delivery"]["deliveryAddress"]["postalCode"] = '69980000';
    	        $order["delivery"]["deliveryAddress"]["neighborhood"] = 'Centro';
    	        $order["delivery"]["deliveryAddress"]["streetNumber"] = '30';
    	        $order["delivery"]["deliveryAddress"]["streetName"] = 'Rua Rui Barbosa';
    	        $order["delivery"]["deliveryAddress"]["complement"] = '';
    	        $order["delivery"]["deliveryAddress"]["reference"] = '';
    	        $consumer["CDMUNICIPIO"] = $this->getTestCityCode();
    	        $consumer["CDBAIRRO"] = $this->getTestNeighborhoodCode($consumer["CDMUNICIPIO"]);
    	    } else if(!$this->validateDeliveryAddress(strtoupper($order["delivery"]["deliveryAddress"]["state"]))){
    	        throw new \Exception("Endereco de entrega inválido.");
    	    }
    	} else if($order["orderType"] === "INDOOR") {
    	    $IDRETBALLOJA = "C";
    	} else {
    	    $IDRETBALLOJA = "S";
    	}
    	
    	$consumerName = strtoupper($this->removeAcentos($order["customer"]["name"]));
    	
    	$params = [
    	    ":CDFILIAL"        => $generalParams["CDFILIAL"],
            ":NRVENDAREST"     => substr(($order["NRVENDAREST"]), 0, 10),
            ":NRCOMANDA"       => substr(($order["NRCOMANDA"]), 0, 10),
            ":NRCOMANDAEXT"    => substr(($order["displayId"]), 0, 50),
            ":CDLOJA"          => $generalParams["CDLOJA"],
            ":DSCOMANDA"       => "DLV_".$order["NRCOMANDA"],
            ":IDSTCOMANDA"     => substr(($order["IDSTCOMANDA"]), 0, 10),
            ":SGSEXOCON"       => "M",
            ":TXMOTIVCANCE"    => "",
            ":VRACRCOMANDA"    => !$removeDeliveryFee ? $this->tofloat($order["total"]["deliveryFee"]) : 0,
            ":CDPAIS"          => "0055",
            ":SGESTADO"        => strtoupper($order["delivery"]["deliveryAddress"]["state"]), 
            ":CDMUNICIPIO"     => $consumer["CDMUNICIPIO"],
            ":NRCEPCONSCOMAND" => substr(($order["delivery"]["deliveryAddress"]["postalCode"]), 0, 15),
            ":CDBAIRRO"        => $consumer["CDBAIRRO"],
            ":DSBAIRRO"        => substr($order["delivery"]["deliveryAddress"]["neighborhood"], 0, 100),
            ":NRENDECONSCOMAN" => substr((empty($order["delivery"]["deliveryAddress"]["streetNumber"]) ? "" : $order["delivery"]["deliveryAddress"]["streetNumber"]), 0, 10),
            ":DSENDECONSCOMAN" => substr($order["delivery"]["deliveryAddress"]["streetName"], 0, 50),
            ":DSCOMPLENDCOCOM" => substr(((isset($order["delivery"]["deliveryAddress"]["complement"]) && !empty($order["delivery"]["deliveryAddress"]["complement"])) ? $order["delivery"]["deliveryAddress"]["complement"] : ""), 0, 200),
            ":DSREFENDCONSCOM" => substr(((isset($order["delivery"]["deliveryAddress"]["reference"]) && !empty($order["delivery"]["deliveryAddress"]["reference"])) ? $order["delivery"]["deliveryAddress"]["reference"] : ""), 0, 2000),
            ":DSOBSCOMANDA"    => substr((isset($generalParams['DSOBSCOMANDA']) ? $generalParams['DSOBSCOMANDA'] : ""), 0, 2000),
            ":IDORGCMDVENDA"   => "DLV_IFO", 
            ":IDRETBALLOJA"    => $IDRETBALLOJA,
            ":NRPEDMOBILE"     => substr(("IFO".$order["id"]), 0, 50),
            ":NRLATPEDIDO"     => substr((isset($order["delivery"]["deliveryAddress"]["coordinates"]["latitude"]) ? $order["delivery"]["deliveryAddress"]["coordinates"]["latitude"] : ""), 0, 50),
            ":NRLONGPEDIDO"    => substr((isset($order["delivery"]["deliveryAddress"]["coordinates"]["longitude"]) ? $order["delivery"]["deliveryAddress"]["coordinates"]["longitude"] : ""), 0, 50),
            ":DSAREAATEND"     => $consumer["DSAREAATEND"],
            ":IDSINCAGENDA"    => substr(($order["IDSINCAGENDA"]), 0, 1),
            ":DTHRAGENDADA"    => $order["DTHRAGENDADA"],
            ":IDCOMANDAEXT"    => substr(($order["id"]), 0, 64),
            ":IDENTRPLAT"      => substr(($generalParams['deliveredBy']), 0, 1),
            ":NRCONTATOCON"    => substr((isset($order["customer"]["phone"]["number"]) ? $order["customer"]["phone"]["number"] : ""), 0, 32),
            ":DSCONSUMIDOR"    => substr((isset($consumerName) ? $consumerName : ""), 0, 50),
            ":IDIMPCUPEXP"     => $order['IDIMPCUPEXP']
    	];
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    private function getTestCityCode()
    {
        $sql = "
            SELECT CDMUNICIPIO 

    	      FROM MUNICIPIO

    	     WHERE UPPER(SGESTADO)  = :SGESTADO
               AND CDPAIS           = :CDPAIS
    		   AND NVL( (SELECT CDMUNICIPIO
    		            
    		              FROM MUNICIPIO
    		           
    		             WHERE UPPER(NMMUNICIPIO)   = :NMMUNICIPIO
                           AND UPPER(SGESTADO)      = :SGESTADO
                           AND CDPAIS               = :CDPAIS
    		               AND ROWNUM               = 1),
    		            (SELECT CDMUNICIPIO
    		            
    		               FROM MUNICIPIO
    		           
    		              WHERE UPPER(NMMUNICIPIO) LIKE '%'||:NMMUNICIPIO||'%'
                            AND UPPER(SGESTADO)     = :SGESTADO
                            AND CDPAIS              = :CDPAIS
    		                AND ROWNUM              = 1)) = CDMUNICIPIO";
    	
    	$params = [
    	    ":CDPAIS"      => "0055",
    	    ":SGESTADO"    => "AC",
    	    ":NMMUNICIPIO" => "CRUZEIRO DO SUL"
    	];
    	
    	$result = $this->connection->fetchAll($sql, $params);
    	
    	if (empty($result)) return null;
    	
    	return $result[0]["CDMUNICIPIO"];
    }
    
    private function getTestNeighborhoodCode($cityCode)
    {
        $sql = "
            SELECT CDBAIRRO

              FROM BAIRRO

             WHERE CDPAIS          = :CDPAIS

               AND SGESTADO        = :SGESTADO
               AND CDMUNICIPIO     = :CDMUNICIPIO
               AND UPPER(NMBAIRRO) = UPPER(:NMBAIRRO)";
        
        $params = [
            ":CDPAIS"      => "0055",
            ":SGESTADO"    => "AC",
            ":CDMUNICIPIO" => $cityCode,
            ":NMBAIRRO"    => "CENTRO"
        ];
        
        $result = $this->connection->fetchAll($sql, $params);
        
        if (empty($result)) 
            return null;
        
        return $result[0]["CDBAIRRO"];
    }
    
    private function validateDeliveryAddress($deliveryState)
    {
        $validStates = [
            'MG',
            'AM',
            'RO',
            'AC',
            'PA',
            'RR',
            'MA',
            'PB',
            'CE',
            'PI',
            'RN',
            'SE',
            'AL',
            'GO',
            'MT',
            'MS',
            'BA',
            'ES',
            'RJ',
            'SP',
            'AP',
            'SC',
            'PR',
            'RS',
            'DF',
            'PE',
            'TO'
        ];
        
        if(in_array($deliveryState, $validStates)){
            return true;
        } else{
            return false;
        }
    }
    
    public function insertCashMoviment($order, $value, $generalParams, $consumer)
    {
        $sql = "
            INSERT INTO MOVCAIXADLV
        		(CDFILIAL, NRVENDAREST, NRSEQMOVDLV, IDTIPOMOVIVEDLV,
        		VRMOVIVENDDLV, CDTIPORECE, CDCLIENTE, CDCONSUMIDOR,
        		NRCARTBANCODLV)
        	VALUES
        		(:CDFILIAL, :NRVENDAREST, :NRSEQMOVDLV, :IDTIPOMOVIVEDLV,
        		:VRMOVIVENDDLV, :CDTIPORECE, :CDCLIENTE, :CDCONSUMIDOR,
        		:NRCARTBANCODLV)";

        $params = [
            ":CDFILIAL"        => $generalParams["CDFILIAL"],
	        ":NRVENDAREST"     => $order["NRVENDAREST"],
	        ":NRSEQMOVDLV"     => $order["NRSEQMOVDLV"],
	        ":IDTIPOMOVIVEDLV" => "E",
	        ":CDCLIENTE"       => $consumer["CDCLIENTE"],
	        ":CDCONSUMIDOR"    => $consumer["CDCONSUMIDOR"],
	        ":CDTIPORECE"      => $order["CDTIPORECE"],
	        ":VRMOVIVENDDLV"   => $this->tofloat($value),
	        ":NRCARTBANCODLV"  => null
        ];

        $this->connection->executeQuery($sql, $params);
    }
    
    public function insertChange($order, $value, $generalParams, $consumer, $movCaixa, $cdTipoRece)
    {

        $sql = "
            INSERT INTO MOVCAIXADLV
        		(CDFILIAL, NRVENDAREST, NRSEQMOVDLV, IDTIPOMOVIVEDLV,
        		VRMOVIVENDDLV, CDTIPORECE, CDCLIENTE, CDCONSUMIDOR,
        		NRCARTBANCODLV)
        	VALUES
        		(:CDFILIAL, :NRVENDAREST, :NRSEQMOVDLV, :IDTIPOMOVIVEDLV,
        		:VRMOVIVENDDLV, :CDTIPORECE, :CDCLIENTE, :CDCONSUMIDOR,
        		:NRCARTBANCODLV)";
        
        $params = [
            ":CDFILIAL"        => $generalParams["CDFILIAL"],
	        ":NRVENDAREST"     => $order["NRVENDAREST"],
	        ":NRSEQMOVDLV"     => $movCaixa,
	        ":IDTIPOMOVIVEDLV" => "S",
	        ":CDCLIENTE"       => $consumer["CDCLIENTE"],
	        ":CDCONSUMIDOR"    => $consumer["CDCONSUMIDOR"],
	        ":CDTIPORECE"      => $cdTipoRece,
	        ":VRMOVIVENDDLV"   => $this->toFloat($value),
	        ":NRCARTBANCODLV"  => null
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function isDuplicated($generalParams, $order)
    {
        $sql = "
            SELECT C.NRPEDMOBILE

    		  FROM COMANDAVEN C

    		  JOIN VENDAREST V
    		    ON V.NRVENDAREST   = C.NRVENDAREST
    		   AND V.CDFILIAL      = C.CDFILIAL

    		 WHERE C.NRPEDMOBILE   = :NRPEDMOBILE
    		   AND V.CDFILIAL      = :CDFILIAL
    		   AND DTHRABERMESA    >= SYSDATE - 1
        ";
        
        $params = [
            ":CDFILIAL"    => $generalParams["CDFILIAL"],
            ":NRPEDMOBILE" => "SHU".$order["id"]
        ];
        
        $result = $this->connection->fetchAll($sql, $params);
        
        return !empty($result);
    }

    public function validateOperator($generalParams)
    {
        $sql = "
            SELECT CDOPERADOR 
              FROM FILIOPER
             WHERE CDOPERADOR   = :CDOPERADOR
               AND CDFILIAL     = :CDFILIAL
        ";

        $params = [
            ":CDFILIAL"   => $generalParams["CDFILIAL"],
            ":CDOPERADOR" => $generalParams["CDOPERADOR"]
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
        
    }

    public function fetchIsCashierSinc($filial)
    {
        $sql = "
            SELECT IDSINCCAIXADLV 
              FROM CAIXA
             WHERE CDFILIAL = :CDFILIAL 
               AND UPPER(IDSINCCAIXADLV) = 'S'
        ";

        $params = [
            ":CDFILIAL" => $filial
        ];

        return $this->connection->fetchAssoc($sql,$params);
    }

    private function tofloat($num)
    {
        $dotPos = strrpos($num, '.');
        $commaPos = strrpos($num, ',');
        $sep = (($dotPos > $commaPos) && $dotPos) ? $dotPos : 
            ((($commaPos > $dotPos) && $commaPos) ? $commaPos : false);
       
        if (!$sep) {
            return floatval(preg_replace("/[^0-9]/", "", $num));
        } 
    
        $tmp = floatval(
            preg_replace("/[^0-9]/", "", substr($num, 0, $sep)) . '.' .
            preg_replace("/[^0-9]/", "", substr($num, $sep+1, strlen($num)))
        );

	    return str_replace(".", ",", $tmp);
    }

    private function getCurrentTimezone($state)
    {
        $timezones = [
            "AC" => "America/Rio_branco",   "AL" => "America/Maceio",
            "AP" => "America/Belem",        "AM" => "America/Manaus",
            "BA" => "America/Bahia",        "CE" => "America/Fortaleza",
            "DF" => "America/Fortaleza",    "ES" => "America/Fortaleza",
            "GO" => "America/Fortaleza",    "MA" => "America/Fortaleza",
            "MT" => "America/Cuiaba",       "MS" => "America/Campo_Grande",
            "MG" => "America/Fortaleza",    "PR" => "America/Fortaleza",
            "PB" => "America/Fortaleza",    "PA" => "America/Belem",
            "PE" => "America/Recife",       "PI" => "America/Fortaleza",
            "RJ" => "America/Fortaleza",    "RN" => "America/Fortaleza",
            "RS" => "America/Fortaleza",    "RO" => "America/Porto_Velho",
            "RR" => "America/Boa_Vista",    "SC" => "America/Fortaleza",
            "SE" => "America/Maceio",       "SP" => "America/Fortaleza",
            "TO" => "America/Araguaia",    
        ];
       
        return $timezones[$state];
    }
    
    // Prevents teknisa from crazy discount logics by partners ¯\_(ツ)_/¯
    
    public function validateOrder($cdFilial, $nrVendaRest, $pizzaOddFlavor = false, $retest = false)
    {
        $taxSQL = "
            SELECT VRACRCOMANDA
              FROM COMANDAVEN
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
        ";

        $itemsSQL = "
            SELECT QTPRODCOMVEN, VRPRECCOMVEN, VRDESCCOMVEN
              FROM ITCOMANDAVEN
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
        ";

        $entradasSQL = "
            SELECT VRMOVIVENDDLV FROM MOVCAIXADLV
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
               AND IDTIPOMOVIVEDLV  = 'E'
        ";

        $saidasSQL = "
            SELECT VRMOVIVENDDLV FROM MOVCAIXADLV
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
               AND IDTIPOMOVIVEDLV  = 'S'
        ";
        
        $params = [
            ":CDFILIAL"     => $cdFilial,
            ":NRVENDAREST"  => $nrVendaRest
        ];

        $taxData      = $this->connection->fetchAssoc($taxSQL, $params);
        $itemsData    = $this->connection->fetchAll($itemsSQL, $params);
        $entradasData = $this->connection->fetchAll($entradasSQL, $params);
        $saidasData   = $this->connection->fetchAll($saidasSQL, $params);

        $taxSum      = isset($taxData['VRACRCOMANDA']) ? (string)$this->formatNumber($taxData['VRACRCOMANDA']) : "0.00";
        $itemsSum    = "0.00";
        $entradasSum = "0.00";
        $saidasSum   = "0.00";

        foreach($itemsData as $itemData) {
            $mul      = bcmul($this->formatNumber($itemData['VRPRECCOMVEN']), $this->formatNumber($itemData['QTPRODCOMVEN']), 3);
            $sub      = bcsub($mul, $this->formatNumber($itemData['VRDESCCOMVEN']), 3);
            $itemsSum = bcadd($itemsSum, $sub, 3);
        }
        $itemsSum = bcadd($itemsSum, '0.00', 2);

        foreach($entradasData as $entradaData) {
            $entradasSum = bcadd($entradasSum, $this->formatNumber($entradaData['VRMOVIVENDDLV']), 2);
        }

        foreach($saidasData as $saidaData) {
            $saidasSum = bcadd($saidasSum, $this->formatNumber($saidaData['VRMOVIVENDDLV']), 2);
        }

        $orderSum    = bcadd($itemsSum, $taxSum, 2);
        $paymentsSum = bcsub($entradasSum, $saidasSum, 2);

        if(!$pizzaOddFlavor) {
            if(bccomp($orderSum, $paymentsSum, 2) != 0) {
                if((bccomp($orderSum, $paymentsSum, 2) == 1)) {
                    $diff = bcsub($orderSum, $paymentsSum, 2);
                    $type = 'D';
                } else {
                    $diff = bcsub($paymentsSum,$orderSum, 2);
                    $type = 'A';
                }

                if ((!$retest) && (bccomp($diff, '0.00', 2) == 1) && ((bccomp($diff, '0.05', 2) == 0) || (bccomp($diff, '0.05', 2) == -1))) {
                    $this->applyFiveCentsFix($cdFilial, $nrVendaRest, $diff, $type);
                    $this->validateOrder($cdFilial, $nrVendaRest, $pizzaOddFlavor, true);
                } else {
                    throw new Exception\ParamsException("
                        Não foi possível processar o pedido. Existe uma divergencia entre os itens do pedido e o valor pago 1.<br>
                        Valor dos itens: {$orderSum}.<br>
                        Valor pago: {$paymentsSum}.");  
                }
            }
        } else {
            if(bccomp($paymentsSum, $orderSum, 2) == -1){
                if(bcsub($orderSum, $paymentsSum, 2) != "0.01"){
                    throw new Exception\ParamsException("
                        Não foi possível processar o pedido. Existe uma divergencia entre os itens do pedido e o valor pago 2.<br>
                        Valor dos itens: {$orderSum}.<br>
                        Valor pago: {$paymentsSum}.");
                }
            } else if(bccomp($paymentsSum, $orderSum, 2) == 1){
                if(bcsub($paymentsSum, $orderSum, 2) != "0.01"){
                    throw new Exception\ParamsException("
                        Não foi possível processar o pedido. Existe uma divergencia entre os itens do pedido e o valor pago 3.<br>
                        Valor dos itens: {$orderSum}.<br>
                        Valor pago: {$paymentsSum}.");
                }
                
            }
        }
    }
    
    private function formatNumber($number)
    {
        return str_replace(",", ".", $number);
    }
    
    private function removeAcentos($texto)
    {
        return preg_replace(array("/(á|à|ã|â|ä)/","/(Á|À|Ã|Â|Ä)/","/(é|è|ê|ë)/","/(É|È|Ê|Ë)/","/(í|ì|î|ï)/","/(Í|Ì|Î|Ï)/","/(ó|ò|õ|ô|ö)/","/(Ó|Ò|Õ|Ô|Ö)/","/(ú|ù|û|ü)/","/(Ú|Ù|Û|Ü)/","/(ñ)/","/(Ñ)/","/(ç)/","/(Ç)/"),explode(" ","a A e E i I o O u U n N c C"),$texto);
    }
    
    private function applyFiveCentsFix($cdFilial, $nrVendaRest, $diff, $type) {
         $itemsSQL = "
            SELECT NRPRODCOMVEN, QTPRODCOMVEN, VRPRECCOMVEN, VRDESCCOMVEN
              FROM ITCOMANDAVEN
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
        ";
        
        $params = [
            ":CDFILIAL"     => $cdFilial,
            ":NRVENDAREST"  => $nrVendaRest
        ];
        
        $itemsData = $this->connection->fetchAll($itemsSQL, $params);
        
        $idxToFix = null;
        $newValueWithoutDiscount = 0;
        
        foreach($itemsData as $itemData) {
            $mul      = bcmul($this->formatNumber($itemData['VRPRECCOMVEN']), $this->formatNumber($itemData['QTPRODCOMVEN']), 3);
            $valueAbs = bcsub($mul, $this->formatNumber($itemData['VRDESCCOMVEN']), 3);
            
            if($type == 'A') {
                $newValueWithoutDiscount = bcadd($this->formatNumber($itemData['VRPRECCOMVEN']), $diff, 2);
            } else {
                $newValueWithoutDiscount = bcsub($this->formatNumber($itemData['VRPRECCOMVEN']), $diff, 2);
            }
            
            if(($itemData['VRDESCCOMVEN'] == 0)) {
                $idxToFix = $itemData['NRPRODCOMVEN'];
            } else {
                $newValue = bcsub($newValueWithoutDiscount, $this->formatNumber($itemData['VRDESCCOMVEN']), 2);
             
                if(($itemData['QTPRODCOMVEN'] == 1) && ((bccomp($newValue, '0.01', 2) == 0) || (bccomp($newValue, '0.01', 2) == 1))) {
                    $idxToFix = $itemData['NRPRODCOMVEN'];
                }
            }
         
            if(!empty($idxToFix)) {
                break;
            }
        }
        
        if(!empty($idxToFix)) {
            $sql = "
                UPDATE ITCOMANDAVEN
                
                  SET VRPRECCOMVEN = :VRPRECCOMVEN
                   
                 WHERE CDFILIAL     = :CDFILIAL
                  AND NRVENDAREST  = :NRVENDAREST 
                  AND NRPRODCOMVEN = :NRPRODCOMVEN
            
            ";
            
            $params = [
                ':CDFILIAL'     => $cdFilial, 
                ':NRVENDAREST'  => $nrVendaRest,
                ':NRPRODCOMVEN' => $idxToFix,
                ':VRPRECCOMVEN' => $this->toFloat($newValueWithoutDiscount)
            ];
            
            $this->connection->executeQuery($sql, $params);
        }
    }
}
