<?php
namespace App\Service;

use App\Exception;

class ConsumerService {
    
    private $util;
    private $consumerFactory;
    private $branchFactory;

    public function __construct (\App\Util\Util $util, 
                                 \App\Factory\ConsumerFactory $consumerFactory,
                                 \App\Factory\BranchFactory $branchFactory) {

        $this->util             = $util;
        $this->consumerFactory  = $consumerFactory;
        $this->branchFactory    = $branchFactory;
    }
    
    public function processConsumer($generalParams, $order) {

        $consumer = $this->consumerFactory->getStandardConsumer($generalParams["CDFILIAL"]);

        $consumer["CDBAIRRO"] = $this->consumerFactory->getNeighborhoodCode($order["entrega_estado"], $order["entrega_cidade"], $order["entrega_bairro"]);

        if (empty($consumer["CDMUNICIPIO"])) 
            $consumer["CDMUNICIPIO"] = $this->consumerFactory->getCityCode($order["entrega_estado"], $order["entrega_cidade"]);

        if (empty($consumer["CDMUNICIPIO"]))
            $consumer["CDMUNICIPIO"] = $this->branchFactory->getBranchCityCode($generalParams["CDFILIAL"]);
            
        $consumer["DSAREAATEND"] = "";
            
        return $consumer;
    }

    private function getDeliveryAreas($locationCoords) {
        
        $areas = $this->branchFactory->getDeliveryAreas();
        if(!$areas['error']) {
            $inside = [];

            $areas = $areas['areas'];
            foreach($areas as $cordinates) {
                $stringCoordinates   = $cordinates["COORDAREA"];
                $stringCoordinates   = str_replace("['","",$stringCoordinates);
                $stringCoordinates   = str_replace("']","",$stringCoordinates);
                $stringCoordinates   = str_replace("','",'$',$stringCoordinates);
                $polyCoordinates     = [];
                $explodedCoordinates = explode('$',$stringCoordinates);  
                
                foreach($explodedCoordinates as $latLongStr) {
                    $latLong = explode(',',$latLongStr);
                    array_push($polyCoordinates, $latLong);
                } 
                
                $inside = $this->inside($locationCoords, $polyCoordinates);
               
                if($inside) {
                    $inside = [
                        "INSIDE"    => $inside,
                        "NAMEAREA"  => $cordinates["DSAREAATEND"],
                    ];

                    return $inside;    
                }
            }

            return $inside;
        } else {
            throw new \Exception($areas["message"], 992); 
        }
    }
    
    private function inside($point, $vs) {
        $x = (float)$point["latitude"];
        $y = (float)$point["longitude"];

        $inside = false;
        for ($i = 0, $j = count($vs) - 1; $i < count($vs); $j = $i++) {
            $xi = (float)$vs[$i][0];
            $yi = (float)$vs[$i][1];
            $xj = (float)$vs[$j][0];
            $yj = (float)$vs[$j][1];

            $intersect = (($yi > $y) != ($yj > $y)) && ($x < ($xj - $xi) * ($y - $yi) / ($yj - $yi) + $xi);
            if ($intersect) 
                $inside = !$inside;
        }
        
        return $inside;
    }
}
